# == Define: sys::inifile
#
# Creates a simple initialization file from the given config hash.  This
# is different from the forge module `ini_file` which specifies a specific
# setting *within* an existing ini file.
#
# === Parameters
#
# Technically only the required parameter is the title of the resource (the
# path to the configuration file).  But it will be empty without supplying
# anything in the configuration hash.
#
# [*config*]
#  Configuration hash, defaults to {}.  Keys are sections, values are hashes of
#  setting/value pairs.  Sections and settings within sections are sorted in the
#  output.
#
# [*backup*]
#  The backup parameter for the ini file resource, undefined by default.
#
# [*header*]
#  Header to use for the ini file, including the comment character.
#  Defaults to: "# This file was autogenerated by sys::inifile."
#
# [*indent*]
#  The number of spaces to use before listing each configuration
#  setting in a section.  Defaults to 0.
#
# [*owner*]
#  The file system owner of the ini file, defaults to 'root'.
#
# [*group*]
#  The file system group of the ini file, defaults to 'root'.
#
# [*mode*]
#  The file system mode of the ini file, defaults to '0644'.
#
# [*template*]
#  Advanced usage only.  The template to use to generate the ini file,
#  defaults to 'sys/inifile/inifile.erb'.
#
# === Example
#
# To generate '/etc/foo.ini' to look like this:
#
#   # This is foo.ini.
#   [bar]
#       setting1 = value1
#   [foo]
#       setting2 = value2
#
# Use the following:
#
#   sys::inifile { '/etc/foo.ini':
#     config  => {
#       'bar' => {
#         'setting1' => 'value1',
#        },
#       'foo' => {
#         'setting2' => 'value2',
#       },
#     },
#     header => '# This is foo.ini.',
#     indent => 4,
#   }
#
define sys::inifile(
  $config   = {},
  $backup   = undef,
  $header   = '# This file was autogenerated by sys::inifile.',
  $indent   = 0,
  $owner    = 'root',
  $group    = 'root',
  $mode     = '0644',
  $template = 'sys/inifile/inifile.erb',
) {
  validate_absolute_path($title)
  validate_hash($config)
  validate_integer($indent)

  file { $title:
    ensure  => file,
    backup  => $backup,
    owner   => $owner,
    group   => $group,
    mode    => $mode,
    content => template($template),
  }
}
